@@ -2,7 +2,7 @@
 
 {-|
 Module      : Database.MySQL.Protocol.Packet
-Description : MySQL packet type
+Description : MySQL packet type and various helpers.
 Copyright   : (c) Winterland, 2016
 License     : BSD
 Maintainer  : drkoster@qq.com
@@ -82,16 +82,9 @@ decodeFromPacket = getFromPacket get
 {-# INLINE decodeFromPacket #-}
 
 getFromPacket :: Get a -> Packet -> IO a
-getFromPacket g (Packet _ _ body) =
-    case body of
-        (L.Chunk bs lbs) -> case pushEndOfInput (pushChunks (parse g bs) lbs) of
-            Done _  _ r             -> return r
-            Fail buf offset errmsg  -> throwIO (DecodePacketFailed buf offset errmsg)
-            _ -> error "getFromPacket: impossible error!"
-        L.Empty -> case pushEndOfInput (parse g B.empty) of
-            Done _  _ r             -> return r
-            Fail buf offset errmsg  -> throwIO (DecodePacketFailed buf offset errmsg)
-            _ -> error "getFromPacket: impossible error!"
+getFromPacket g (Packet _ _ body) = case parseDetailLazy g body of
+    Left  (buf, offset, errmsg) -> throwIO (DecodePacketFailed buf offset errmsg)
+    Right (_,   _,      r     ) -> return r
 {-# INLINE getFromPacket #-}
 
 data DecodePacketException = DecodePacketFailed ByteString ByteOffset String
@@ -126,7 +119,7 @@ data OK = OK
     } deriving (Show, Eq)
 
 getOK :: Get OK
-getOK = OK <$ skip 1
+getOK = OK <$ skipN 1
            <*> getLenEncInt
            <*> getLenEncInt
            <*> getWord16le
@@ -155,9 +148,9 @@ data ERR = ERR
     } deriving (Show, Eq)
 
 getERR :: Get ERR
-getERR = ERR <$  skip 1
+getERR = ERR <$  skipN 1
              <*> getWord16le
-             <*  skip 1
+             <*  skipN 1
              <*> getByteString 5
              <*> getRemainingByteString
 {-# INLINE getERR #-}
@@ -183,7 +176,7 @@ data EOF = EOF
     } deriving (Show, Eq)
 
 getEOF :: Get EOF
-getEOF = EOF <$  skip 1
+getEOF = EOF <$  skipN 1
              <*> getWord16le
              <*> getWord16le
 {-# INLINE getEOF #-}
